#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""APRS Cursor-on-Target Class Definitions."""

import asyncio

import aprslib.parsing
import pytak

import aprscot

__author__ = "Greg Albrecht W2GMD <oss@undef.net>"
__copyright__ = "Copyright 2021 Greg Albrecht"
__license__ = "Apache License, Version 2.0"
__source__ = "https://github.com/ampledata/aprscot"


class APRSWorker(pytak.MessageWorker):

    """APRS Cursor-on-Target Worker Class."""

    def __init__(self, event_queue: asyncio.Queue, config: dict) -> None:
        super().__init__(event_queue)
        self.config = config

        # APRS Parameters:
        self.passcode = "-1"

        self.callsign: str = config["aprscot"].get(
            "CALLSIGN", aprscot.DEFAULT_APRSIS_CALLSIGN)

        self.aprs_filter: str = config["aprscot"].get(
            "APRSIS_FILTER", aprscot.DEFAULT_APRSIS_FILTER)

        # Figure out APRS Host:
        aprs_host: str = config["aprscot"].get(
            "APRS_HOST", aprscot.DEFAULT_APRSIS_HOST)
        aprs_port: str = config["aprscot"].get(
            "APRS_PORT", aprscot.DEFAULT_APRSIS_PORT)

        if ':' in aprs_host:
            aprs_host, aprs_port = aprs_host.split(':')

        self.aprs_host = aprs_host
        self.aprs_port = aprs_port
        self._logger.info(
            "Using APRS-IS server: %s:%s", self.aprs_host, self.aprs_port)

    async def handle_message(self, message: bytes) -> None:
        """Handles messages from APRS Worker."""
        self._logger.debug("APRS message='%s'", message)

        # Skip control messages from APRS-IS:
        if b"# " in message[:2]:
            self._logger.info("APRS-IS: '%s'", message)
            return

        # Some APRS Frame types are not supported by aprslib yet:
        try:
            aprs_frame = aprslib.parsing.parse(message)
        except aprslib.exceptions.UnknownFormat as exc:
            self._logger.debug(exc)
            self._logger.warning("Unhandled APRS Frame: '%s'", message)
            return

        # self._logger.debug("aprs_frame=%s", aprs_frame)

        event = aprscot.aprs_to_cot(aprs_frame, self.config)
        if not event:
            self._logger.warning(
                "Empty CoT Event for APRS Frame: '%s'", aprs_frame.get("raw"))
            return

        await self._put_event_queue(event)

    async def run(self):  # pylint: disable=arguments-differ
        """Runs this Thread, Reads from Pollers."""
        self._logger.info("Running APRSWorker")

        reader, writer = await asyncio.open_connection(
            self.aprs_host, int(self.aprs_port))

        _login = (
            f"user {self.callsign} pass {self.passcode} vers aprscot v5.0.0")

        if self.aprs_filter:
            self._logger.info("Using APRS Filter: '%s'", self.aprs_filter)
            _login = f"{_login} filter {self.aprs_filter}"
        _login = f"{_login}\r\n"

        b_login = bytes(_login, "UTF-8")
        writer.write(b_login)
        await writer.drain()

        while 1:
            frame = await reader.readline()
            if frame:
                await self.handle_message(frame)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""APRS Cursor-on-Target Gateway Commands."""

import argparse
import asyncio
import configparser
import logging
import sys
import urllib

import pytak

import aprscot

# Python 3.6 support:
if sys.version_info[:2] >= (3, 7):
    from asyncio import get_running_loop
else:
    from asyncio import _get_running_loop as get_running_loop  # NOQA pylint: disable=no-name-in-module


__author__ = 'Greg Albrecht W2GMD <oss@undef.net>'
__copyright__ = 'Copyright 2021 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'
__source__ = 'https://github.com/ampledata/aprscot'


async def main(config):
    """Main program function. Sets Queues, Executes Workers."""
    tx_queue: asyncio.Queue = asyncio.Queue()
    rx_queue: asyncio.Queue = asyncio.Queue()

    cot_url: urllib.parse.ParseResult = urllib.parse.urlparse(
        config["aprscot"].get("COT_URL"))

    # Create our CoT Event Queue Worker
    reader, writer = await pytak.protocol_factory(cot_url)
    write_worker = pytak.EventTransmitter(tx_queue, writer)
    read_worker = pytak.EventReceiver(rx_queue, reader)

    message_worker = aprscot.APRSWorker(tx_queue, config)

    await tx_queue.put(pytak.hello_event("aprscot"))

    done, _ = await asyncio.wait(
        {message_worker.run(), read_worker.run(), write_worker.run()},
        return_when=asyncio.FIRST_COMPLETED)

    for task in done:
        print(f"Task completed: {task}")


def cli():
    """Command Line interface for APRS Cursor-on-Target Gateway."""

    # Get cli arguments:
    parser = argparse.ArgumentParser()

    parser.add_argument("-c", "--CONFIG_FILE", dest="CONFIG_FILE",
                        default="config.ini", type=str)
    namespace = parser.parse_args()
    cli_args = {k: v for k, v in vars(namespace).items() if v is not None}

    # Read config file:
    config = configparser.ConfigParser()

    config_file = cli_args.get("CONFIG_FILE")
    logging.info("Reading configuration from %s", config_file)
    config.read(config_file)

    if sys.version_info[:2] >= (3, 7):
        asyncio.run(main(config), debug=config["aprscot"].getboolean("DEBUG"))
    else:
        loop = asyncio.get_event_loop()
        try:
            loop.run_until_complete(main(config))
        finally:
            loop.close()


if __name__ == "__main__":
    cli()
